@@ -138,11 +138,11 @@ typedef void* GtkWidget;
  */
 void SetDisplayCallback(GtkWidget *widget, gpointer *data)
 {
-	HciRegisters reg;
+	SMMRegisters reg;
 
-	reg.ax = HCI_SET;
-	reg.bx = HCI_VIDEO_OUT;
-	reg.cx = (int) data;
+	reg.eax = HCI_SET;
+	reg.ebx = HCI_VIDEO_OUT;
+	reg.ecx = (int) data;
 	HciFunction(&reg);
 	output = (int) data;
 
@@ -166,16 +166,16 @@ void Deleted(GtkWidget *widget, gpointer *data)
  */
 gint Update(gpointer data)
 {
-	HciRegisters reg;
+	SMMRegisters reg;
 
-	reg.ax = HCI_GET;
-	reg.bx = HCI_VIDEO_OUT;
+	reg.eax = HCI_GET;
+	reg.ebx = HCI_VIDEO_OUT;
 	HciFunction(&reg);
 
-	if (output==(int) reg.cx)
+	if (output==(int) reg.ecx)
 		return TRUE;
 
-	switch (reg.cx) {
+	switch (reg.ecx) {
 		case HCI_INTERNAL:
 			gtk_toggle_button_set_state(
 				GTK_TOGGLE_BUTTON(internal), TRUE);
@@ -187,7 +187,7 @@ gint Update(gpointer data)
 				GTK_TOGGLE_BUTTON(simultaneous), TRUE);
 	}
 
-	output = (int) reg.cx;
+	output = (int) reg.ecx;
 
 	return TRUE;
 }
@@ -263,7 +263,7 @@ void ProcessComandLine(int *argc, char ***argv)
 int main(int argc, char *argv[])
 {
 	int version;
-	HciRegisters reg;
+	SMMRegisters reg;
 #ifndef NO_X11
 	int pid;
 	FILE *str;
@@ -274,18 +274,6 @@ int main(int argc, char *argv[])
 	GSList *radio;
 #endif
 
-	/* get the necessary I/O permissions */
-
-	if (ioperm(0xb2, 1, 1)) {
-		fprintf(stderr, "dispswitch: can't get I/O permissions.\n");
-		return 1;
-	}
-
-        /* drop root priveleges to minimize the risk of running suid root */
-
-        seteuid(getuid());
-        setegid(getgid());
-
 	/* do some quick checks on the laptop */
 
 	if (SciSupportCheck(&version)==1) {
@@ -293,8 +281,8 @@ int main(int argc, char *argv[])
 		return 1;
 	}
 
-	reg.ax = HCI_GET;
-	reg.bx = HCI_VIDEO_OUT;
+	reg.eax = HCI_GET;
+	reg.ebx = HCI_VIDEO_OUT;
 	if (HciFunction(&reg)==HCI_NOT_SUPPORTED) {
 		fprintf(stderr, "dispswitch: this computer is not supported\n");
 		return 1;
