@@ -3,6 +3,10 @@
  * Copyright (c) 1996,97,98  Jonathan A. Buzzard (jonathan@buzzard.org.uk)
  *
  * $Log: fan.c,v $
+ * Revision 3.4  1999/12/01 19:59:23  jab
+ * added daemon option to keep fan turned on when on external power
+ * sorted out the assembler so it compiles and works whatever gcc is used
+ *
  * Revision 3.3  1998/12/30 23:42:45  jab
  * added toggle option to turn the fan off if on and vice versa
  * started move to Hardware Configuration Interface routines
@@ -61,7 +65,7 @@
  *
  */
 
-static char const rcsid[]="$Id: fan.c,v 3.3 1998/12/30 23:42:45 jab Exp jab $";
+static char const rcsid[]="$Id: fan.c,v 3.4 1999/12/01 19:59:23 jab Exp jab $";
 
 
 #include<stdio.h>
@@ -80,222 +84,54 @@ static char const rcsid[]="$Id: fan.c,v 3.3 1998/12/30 23:42:45 jab Exp jab $";
 #include"hci.h"
 #include"sci.h"
 
-enum { DEFAULT, PORTAGE, TECRA };
 enum { ON, OFF, STATUS, AUTO, TOGGLE };
 
 #define PID_FILE _PATH_VARRUN "fan.pid"
 
-int method;
-
-int HciFanOn(int method)
-{
-	unsigned char status;
-
-	switch (method) {
-		case PORTAGE:
-			asm ("pushl %eax\n\t" \
-				"pushl %ebx\n\t" \
-				"pushl %ecx\n\t" \
-				"cli\n\t" \
-				"movb $0xbe,%al\n\t" \
-				"outb %al,$0xe4\n\t" \
-				"inb $0xe5,%al\n\t" \
-				"andb $0xfe,%al\n\t" \
-				"movb %al,%ah\n\t" \
-				"movb $0xbe,%al\n\t" \
-				"outb %al,$0xe4\n\t" \
-				"movb %ah,%al\n\t" \
-				"outb %al,$0xe5\n\t" \
-				"sti\n\t" \
-				"popl %ecx\n\t" \
-				"popl %ebx\n\t" \
-				"popl %eax\n");
-			status = 0x00;
-			break;
-		case TECRA:
-			asm ("pushl %eax\n\t" \
-				"pushl %ebx\n\t" \
-				"pushl %ecx\n\t" \
-				"cli\n\t" \
-				"movw $0x00e4,%dx\n\t" \
-				"movb $0xe0,%al\n\t" \
-				"outb %al,%dx\n\t" \
-				"incw %dx\n\t" \
-				"inb %dx,%al\n\t" \
-				"orw $0x0001,%ax\n\t" \
-				"decw %dx\n\t" \
-				"movb %al,%ah\n\t" \
-				"movb $0xe0,%al\n\t" \
-				"outw %ax,%dx\n\t" \
-				"sti\n\t" \
-				"popl %ecx\n\t" \
-				"popl %ebx\n\t" \
-				"popl %eax\n");
-			status = 0x00;
-			break;
-		default:
-			asm ("pushl %%eax\n\t" \
-				"pushl %%ebx\n\t" \
-				"pushl %%ecx\n\t" \
-				"movw $0xffff,%%ax\n\t" \
-				"movw $0x0004,%%bx\n\t" \
-				"movw $0x0001,%%cx\n\t" \
-				"inb $0xb2,%%al\n\t" \
-				"movb %%ah,%0\n" \
-				"popl %%ecx\n\t" \
-				"popl %%ebx\n\t" \
-				"popl %%eax\n" \
-				:"=m" (status) : : "memory" );
-			break;
-	}
-
-	return (int) status;
-}
-
-int HciFanOff(int method)
-{
-	unsigned char status;
-
-	switch (method) {
-		case PORTAGE:
-			asm ("pushl %eax\n\t" \
-				"pushl %ebx\n\t" \
-				"pushl %ecx\n\t" \
-				"cli\n\t" \
-				"movb $0xbe,%al\n\t" \
-				"outb %al,$0xe4\n\t" \
-				"inb $0xe5,%al\n\t" \
-				"orb $0x01,%al\n\t" \
-				"movb %al,%ah\n\t" \
-				"movb $0xbe,%al\n\t" \
-				"outb %al,$0xe4\n\t" \
-				"movb %ah,%al\n\t" \
-				"outb %al,$0xe5\n\t" \
-				"sti\n\t" \
-				"popl %ecx\n\t" \
-				"popl %ebx\n\t" \
-				"popl %eax\n");
-			status = 0x00;
-			break;
-		case TECRA:
-			asm ("pushl %eax\n\t" \
-				"pushl %ebx\n\t" \
-				"pushl %ecx\n\t" \
-				"cli\n\t" \
-				"movw $0x00e4,%dx\n\t" \
-				"movb $0xe0,%al\n\t" \
-				"outb %al,%dx\n\t" \
-				"incw %dx\n\t" \
-				"inb %dx,%al\n\t" \
-				"andw $0xfffe,%ax\n\t" \
-				"decw %dx\n\t" \
-				"movb %al,%ah\n\t" \
-				"movb $0xe0,%al\n\t" \
-				"outw %ax,%dx\n\t" \
-				"sti\n\t" \
-				"popl %ecx\n\t" \
-				"popl %ebx\n\t" \
-				"popl %eax\n");
-			status = 0x00;
-			break;
-		default:
-			asm ("pushl %%eax\n\t" \
-				"pushl %%ebx\n\t" \
-				"pushl %%ecx\n\t" \
-				"movw $0xffff,%%ax\n\t" \
-				"movw $0x0004,%%bx\n\t" \
-				"movw $0x0000,%%cx\n\t" \
-				"inb $0xb2,%%al\n\t" \
-				"movb %%ah,%0\n" \
-				"popl %%ecx\n\t" \
-				"popl %%ebx\n\t" \
-				"popl %%eax\n" \
-				:"=m" (status) : : "memory" );
-			break;
-	}
-
-	return (int) status;
-}
-
-int HciFanStatus(int method)
-{
-	int status;
-	unsigned char ah;
-	unsigned short ax,cx;
-
-	switch(method) {
-		case PORTAGE:
-			asm ("pushl %%eax\n\t" \
-				"pushl %%ebx\n\t" \
-				"pushl %%ecx\n\t" \
-				"cli\n\t" \
-				"movb $0xbe,%%al\n\t" \
-				"outb %%al,$0xe4\n\t" \
-				"inb $0xe5,%%al\n\t" \
-				"andw $0x0001,%%ax\n\t" \
-				"movw %%ax,%0\n\t" \
-				"sti\n\t" \
-				"popl %%ecx\n\t" \
-				"popl %%ebx\n\t" \
-				"popl %%eax\n" \
- 				:"=m" (ax) : : "memory" );
-			status = (ax==0x00) ? 0x01:0x00;
-			break;
-		case TECRA:
-			asm ("pushl %%eax\n\t" \
-				"pushl %%ebx\n\t" \
-				"pushl %%ecx\n\t" \
-				"cli\n\t" \
-				"movw $0x00e4,%%dx\n\t" \
-				"movb $0xe0,%%al\n\t" \
-				"outb %%al,%%dx\n\t" \
-				"incw %%dx\n\t" \
-				"inb %%dx,%%al\n\t" \
-				"andw $0x0001,%%ax\n\t"
-				"movw %%ax,%0\n\t" \
-				"sti\n\t" \
-				"popl %%ecx\n\t" \
-				"popl %%ebx\n\t" \
-				"popl %%eax\n" \
-				:"=m" (ax) : : "memory" );
-			status = (int) ax;
-			break;
-		default:
-			asm ("pushl %%eax\n\t" \
-				"pushl %%ebx\n\t" \
-				"pushl %%ecx\n\t" \
-				"movw $0xfefe,%%ax\n\t" \
-				"movw $0x0004,%%bx\n\t" \
-				"movw $0x0000,%%cx\n\t" \
-				"inb $0xb2,%%al\n\t" \
-				"movw %%cx,%0\n\t" \
-				"movb %%ah,%1\n" \
-				"popl %%ecx\n\t" \
-				"popl %%ebx\n\t" \
-				"popl %%eax\n" \
- 				:"=m" (cx), "=m" (ah) : : "memory" );
-			status = (ah==0x00) ? (int) cx & 0x0001 : -1*(int) ah;
-			break;
-	}
-
-	return status;
-}
 
 void FanOn(char *name)
 {
-	HciFanOn(method);
+	SMMRegisters reg;
+
+	reg.eax = HCI_SET;
+	reg.ebx = HCI_FAN;
+	reg.ecx = HCI_ENABLE;
+	HciFunction(&reg);
+
 	syslog(LOG_INFO, "cooling fan turned on by %s", name);
 	return;
 }
 
+
 void FanOff(char *name)
 {
-	HciFanOff(method);
+	SMMRegisters reg;
+
+	reg.eax = HCI_SET;
+	reg.ebx = HCI_FAN;
+	reg.ecx = HCI_DISABLE;
+	HciFunction(&reg);
+
 	syslog(LOG_INFO, "cooling fan turned off by %s", name);
 	return;
 }
 
 
+int FanStatus(void)
+{
+	SMMRegisters reg;
+
+	reg.eax = HCI_GET;
+	reg.ebx = HCI_FAN;
+	HciFunction(&reg);
+
+	if (reg.eax!=HCI_SUCCESS)
+		return -1;
+	else
+		return (int) reg.ecx;
+}
+
+
 static void Catch(int sig)
 {
 	syslog(LOG_INFO, "Exiting" );
@@ -341,6 +177,7 @@ void Daemon(void)
 {
 	int pid;
 	FILE *str;
+	SMMRegisters reg;
 
 	/* register some signal handlers */
 
@@ -385,7 +222,12 @@ void Daemon(void)
 	/* enter into an infinite loop */
 
 	for (;;) {
-		if (SciACPower()==SCI_MAINS) HciFanOn(method);
+		if (SciACPower()==SCI_MAINS) {
+			reg.eax = HCI_SET;
+			reg.ebx = HCI_FAN;
+			reg.ecx = HCI_ENABLE;
+			HciFunction(&reg);
+		}
 		sleep(1);
 	}
 
@@ -394,39 +236,18 @@ void Daemon(void)
 
 int main(int argc, char *argv[])
 {
-	int id,opp;
+	int opp;
 	struct passwd *pw;
 	char *name;
 
-	/* get the necessary I/O permissions */
 
-	if (iopl(3)) {
-		fprintf(stderr, "fan: can't get I/O permissions.\n");
-		return 1;
-	}
-
-	/* drop root priveleges to reduce security risk of running suid root */
-/*
-	seteuid(getuid());
-	setegid(getgid());
-*/
 	/* check to see if already running in daemon mode */
 
 	Running();
 
-	/* select a method based on the model we are running on */
-
-	method = DEFAULT;
-	if (HciGetMachineID(&id)==SCI_SUCCESS) {
-		if (id==0xfccb)
-			method = PORTAGE;
-		else if (id==0xfccc)
-			method = TECRA;
-	}
-
 	/* check to make sure laptop has a fan */
 
-	if (HciFanStatus(method)<0x00) {
+	if (FanStatus()<0x00) {
 		fprintf(stderr, "fan: laptop does not have cooling fan.\n");
 		return 1;
 	}
@@ -455,7 +276,7 @@ int main(int argc, char *argv[])
 				opp = ON;
 			}
 		} else if (!strcmp(argv[1], "-t") || !strcmp(argv[1], "--toggle")) {
-			if (HciFanStatus(method)==0x00)
+			if (FanStatus()==0x00)
 				opp = ON;
 			else
 				opp = OFF;
@@ -485,7 +306,7 @@ int main(int argc, char *argv[])
 
 	/* print the current status of the fan */
 
-	if (HciFanStatus(method)==0x00)
+	if (FanStatus()==0x00)
 		printf("Fan is off.\n");
 	else
 		printf("Fan is on.\n");
