@@ -1,6 +1,6 @@
 /* hci.h -- Hardware Configuration Interface
  *
- * Copyright (c) 1998,1999  Jonathan A. Buzzard (jab@hex.prestel.co.uk)
+ * Copyright (c) 1998,1999  Jonathan A. Buzzard (jonathan@buzzard.org.uk)
  *
  * WARNING WARNING WARNING WARNING WARNING WARNING WARNING WARNING WARNING
  *
@@ -11,6 +11,10 @@
  *   not to make any modifications and use it as it stands.
  *
  * $Log: hci.h,v $
+ * Revision 1.2  1999/12/17 12:19:46  jab
+ * added function prototypes for HciFunction and HciFnStatus
+ * added lots of enum types for various settings
+ *
  * Revision 1.1  1999/03/11 20:29:11  jab
  * Initial revision
  *
@@ -38,12 +42,17 @@
 extern "C" {
 #endif
 
+#ifndef SMMREGISTERS
+#define SMMREGISTERS
 typedef struct {
-	unsigned short ax;
-	unsigned short bx;
-	unsigned short cx;
-	unsigned short dx;
-} HciRegisters;
+	unsigned long eax;
+	unsigned long ebx;
+	unsigned long ecx;
+	unsigned long edx;
+	unsigned long esi;
+	unsigned long edi;
+} SMMRegisters;
+#endif
 
 enum {
 	HCI_GET             = 0xfe00,
@@ -52,18 +61,21 @@ enum {
 
 
 enum {
-	HCI_BACKLIGHT       = 0x0002,
-	HCI_AC_ADAPTOR      = 0x0003,
-	HCI_FAN             = 0x0004,
+	HCI_BACKLIGHT        = 0x0002,
+	HCI_AC_ADAPTOR       = 0x0003,
+	HCI_FAN              = 0x0004,
 	HCI_SOFTWARE_SUSPEND = 0x0010,
-	HCI_FLAT_PANEL      = 0x0011,
-	HCI_SYSTEM_EVENT    = 0x0016,
-	HCI_FIR_STATUS      = 0x001b,
-	HCI_VIDEO_OUT       = 0x001c,
-	HCI_HOTKEY_EVENT    = 0x001e,
-	HCI_UNUSED_MEMORY   = 0x0021,
-	HCI_HIBERNATION_INFO = 0x0002d,
-	HCI_HIBERNATION_LBA = 0x002e
+	HCI_FLAT_PANEL       = 0x0011,
+	HCI_SELECT_STATUS    = 0x0014,
+	HCI_SYSTEM_EVENT     = 0x0016,
+	HCI_FIR_STATUS       = 0x001b,
+	HCI_VIDEO_OUT        = 0x001c,
+	HCI_HOTKEY_EVENT     = 0x001e,
+	HCI_UNUSED_MEMORY    = 0x0021,
+	HCI_LOCK_STATUS      = 0x0022,
+	HCI_BOOT_DEVICE      = 0x0026,	
+	HCI_HIBERNATION_INFO = 0x002d,
+	HCI_HIBERNATION_LBA  = 0x002e
 };
 
 
@@ -105,6 +117,26 @@ enum {
 	HCI_VRAM_SIZE       = 0x0002
 };
 
+enum {
+	HCI_BUILT_IN        = 0x0000,
+	HCI_SELECT_INT      = 0x0001,
+	HCI_SELECT_DOCK     = 0x0002,
+	HCI_5INCH_DOCK      = 0x0003
+};
+
+enum {
+	HCI_LOCKED          = 0x0000,
+	HCI_UNLOCKED        = 0x0001
+};
+
+enum {
+	HCI_NOTHING         = 0x0000,
+	HCI_FLOPPY          = 0x0001,
+	HCI_ATAPI           = 0x0002,
+	HCI_IDE             = 0x0003,
+	HCI_BATTERY         = 0x0004
+};
+
 
 /*
  * HCI error codes
@@ -119,18 +151,13 @@ enum {
 };
 
 
-
-
 /*
  * function prototypes
  */
-int HciGet(unsigned short mode, unsigned short *status);
-int HciSet(unsigned short mode, unsigned short status);
-int HciFunction(HciRegisters *reg);
+int HciFunction(SMMRegisters *reg);
 int HciGetBiosVersion(void);
 int HciGetMachineID(int *id);
-int HciFnStatus(void);
-
+int HciGetLCDPanelType(int *resolution, int *type);
 
 #ifdef __cplusplus
 }
