@@ -3,31 +3,35 @@
  * Copyright (c) 1996-98  Jonathan A. Buzzard (jonathan@buzzard.org.uk)
  *
  * $Log: machine.c,v $
+ * Revision 1.2  1999/12/04 13:43:21  jab
+ * changed for the new tuxtime-conf program
+ *
  * Revision 1.1  1999/04/09 17:59:08  jab
  * Initial revision
  *
  *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 675 Mass Ave, Cambridge, MA 02139, USA.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software 
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  */
 
-static const char rcsid[]="$Id: machine.c,v 1.1 1999/04/09 17:59:08 jab Exp jab $";
+static const char rcsid[]="$Id: machine.c,v 1.2 1999/12/04 13:43:21 jab Exp jab $";
 
 
 #include<stdio.h>
 #include<stdlib.h>
+#include<paths.h>
 #include<gtk/gtk.h>
 
 #include"sci.h"
@@ -84,7 +88,7 @@ static int lcdtype,savetype;
 GtkWidget *battery,*hdd,*display,*autooff;
 GtkWidget *speed,*sleep,*lcd,*cooling,*standby;
 GtkWidget *panel,*low,*power,*level,*beep;
-GtkWidget *power;
+GtkWidget *resume,*disable;
 GtkWidget *hourplus,*hourminus,*minuteplus,*minuteminus;
 GtkWidget *alarmtime,*alarmam,*alarmpm,*alarm24;
 GtkObject *slider;
@@ -110,6 +114,8 @@ void SleepCallback(GtkWidget *widget, int data);
 void LCDCallback(GtkWidget *widget, int data);
 void CoolingCallback(GtkWidget *widget, int data);
 void StandbyCallback(GtkWidget *widget, int data);
+void SetSystemOptions(void);
+
 
 enum { STOCKHOLM, ATLANTA, MAUI, LIBRETTO };
 enum { BRIGHT, SUPER, BACKLIGHT };
@@ -146,8 +152,8 @@ static MenuItem batterymaui[] =
 
 static MenuItem batterystockholm[] =
 {
-	{"Full Power", BatteryCallback, (gpointer) SCI_FULL_POWER, NULL},
-	{"Economy", BatteryCallback, (gpointer) SCI_LOW_POWER, NULL},
+	{"Full Power", BatteryCallback, (gpointer) SCI_FULL_LIFE, NULL},
+	{"Economy", BatteryCallback, (gpointer) SCI_ECONOMY, NULL},
 	{"Low Power", BatteryCallback, (gpointer) SCI_LOW_POWER, NULL},
 	{"User Settings", BatteryCallback, (gpointer) SCI_USER_SETTINGS, NULL},
 	{NULL, NULL, NULL, NULL}
@@ -155,8 +161,8 @@ static MenuItem batterystockholm[] =
 
 static MenuItem batteryatlanta[] =
 {
-	{"Normal Life", BatteryCallback, (gpointer) SCI_FULL_POWER, NULL},
-	{"Long Life", BatteryCallback, (gpointer) SCI_LOW_POWER, NULL},
+	{"Normal Life", BatteryCallback, (gpointer) SCI_NORMAL_LIFE, NULL},
+	{"Long Life", BatteryCallback, (gpointer) SCI_LONG_LIFE, NULL},
 	{"User Settings", BatteryCallback, (gpointer) SCI_USER_SETTINGS, NULL},
 	{NULL, NULL, NULL, NULL}
 };
@@ -277,6 +283,8 @@ void ResumeYesCallback(GtkWidget *widget, gpointer *data)
 {
 	SetSetting(SCI_POWER_UP, SCI_RESUME, ERROR_GUI);
 
+	SetSystemOptions();
+
 	gtk_widget_destroy(GTK_WIDGET(data));
 	return;
 }
@@ -357,11 +365,13 @@ int GetMenuItemNumber(MenuItem *menu, int setting)
 {
 	int i;
 
-	for (i=0;menu[i].data!=NULL;i++) {
+	for (i=0;menu[i].label!=NULL;i++) {
 		if ((int) menu[i].data==setting)
 			return i;
 	}
 
+	printf("GetMenuItemNumber failed\n");
+
 	return -1;
 }
 
@@ -374,7 +384,7 @@ void SetBatteryOptions(int mode)
 	int i = 0;
 	struct BatterySettings settings;
 
-	printf("SET BATTERY OPTIONS CALLED %d\n", mode);
+	printf("SET BATTERY OPTIONS CALLED %d %d\n", mode, savetype);
 
 	if (mode==SCI_MAINS) {
 		settings.save = external.save;
@@ -422,28 +432,17 @@ void SetBatteryOptions(int mode)
 	if ((i = GetMenuItemNumber(autooffmenu, settings.autooff))!=-1)
 		gtk_option_menu_set_history(GTK_OPTION_MENU(autooff), i);
 
-	if (libretto) {
-		if ((i = GetMenuItemNumber(standbymenu, settings.standby))!=-1)
-			gtk_option_menu_set_history(GTK_OPTION_MENU(standby), i);
-	}
-
 	if ((i = GetMenuItemNumber(speedmenu, settings.speed))!=-1)
 		gtk_option_menu_set_history(GTK_OPTION_MENU(speed), i);
 
 	if ((i = GetMenuItemNumber(sleepmenu, settings.sleep))!=-1)
 		gtk_option_menu_set_history(GTK_OPTION_MENU(sleep), i);
 
-	if (libretto) {
-		gtk_option_menu_set_history(GTK_OPTION_MENU(lcd), 3-settings.lcd);
-	} else {
-		if ((i = GetMenuItemNumber(lcdbackmenu, settings.lcd))!=-1)
+	if ((i = GetMenuItemNumber(lcdsupermenu, settings.lcd))!=-1)
 			gtk_option_menu_set_history(GTK_OPTION_MENU(lcd), i);
-	}
 
-	if (!libretto) {
-		if ((i = GetMenuItemNumber(coolingmenu, settings.cooling))!=-1)
-			gtk_option_menu_set_history(GTK_OPTION_MENU(cooling), i);
-	}
+	if ((i = GetMenuItemNumber(coolingmenu, settings.cooling))!=-1)
+		gtk_option_menu_set_history(GTK_OPTION_MENU(cooling), i);
 
 	return;
 }
@@ -454,21 +453,16 @@ void SetBatteryOptions(int mode)
  */
 void SetBatterySensitivity(int save)
 {
-printf("BATTERY SENSITIVITY = %d %d\n", save, libretto);
+	printf("BATTERY SENSITIVITY = %d\n", save);
 
 	if (save==SCI_USER_SETTINGS) {
-		if (!libretto) {
-			gtk_widget_set_sensitive(GTK_WIDGET(hdd), TRUE);
-			gtk_widget_set_sensitive(GTK_WIDGET(display), TRUE);
-		}
+		gtk_widget_set_sensitive(GTK_WIDGET(hdd), TRUE);
+		gtk_widget_set_sensitive(GTK_WIDGET(display), TRUE);
 		gtk_widget_set_sensitive(GTK_WIDGET(autooff), TRUE);
 		gtk_widget_set_sensitive(GTK_WIDGET(speed), TRUE);
 		gtk_widget_set_sensitive(GTK_WIDGET(sleep), TRUE);
 		gtk_widget_set_sensitive(GTK_WIDGET(lcd), TRUE);
-		if (libretto)
-			gtk_widget_set_sensitive(GTK_WIDGET(standby), TRUE);
-		else
-			gtk_widget_set_sensitive(GTK_WIDGET(cooling), TRUE);
+		gtk_widget_set_sensitive(GTK_WIDGET(cooling), TRUE);
 	} else {
 		gtk_widget_set_sensitive(GTK_WIDGET(hdd), FALSE);
 		gtk_widget_set_sensitive(GTK_WIDGET(display), FALSE);
@@ -476,10 +470,7 @@ printf("BATTERY SENSITIVITY = %d %d\n", save, libretto);
 		gtk_widget_set_sensitive(GTK_WIDGET(speed), FALSE);
 		gtk_widget_set_sensitive(GTK_WIDGET(sleep), FALSE);
 		gtk_widget_set_sensitive(GTK_WIDGET(lcd), FALSE);
-		if (libretto)
-			gtk_widget_set_sensitive(GTK_WIDGET(standby), FALSE);
-		else			
-			gtk_widget_set_sensitive(GTK_WIDGET(cooling), FALSE);
+		gtk_widget_set_sensitive(GTK_WIDGET(cooling), FALSE);
 	}
 
 	return;
@@ -494,6 +485,11 @@ void PowerSourceCallback(GtkWidget *widget, int data)
 	if (source!=(int) data) {
 	        source = (int) data;
 		SetBatteryOptions(source);
+
+		if (source==SCI_MAINS)
+			SetBatterySensitivity(external.save);
+		else
+			SetBatterySensitivity(internal.save);
 	}
 
 	return;
@@ -505,7 +501,6 @@ void PowerSourceCallback(GtkWidget *widget, int data)
  */
 void BatteryCallback(GtkWidget *widget, int data)
 {
-	SetSetting(SCI_BATTERY_SAVE, data, ERROR_GUI);
 	if (source==SCI_MAINS)
 		external.save = data;
 	else
@@ -520,7 +515,6 @@ void BatteryCallback(GtkWidget *widget, int data)
 
 void HDDCallback(GtkWidget *widget, int data)
 {
-	SetSetting(SCI_HDD_AUTO_OFF, data, ERROR_GUI);
 	if (source==SCI_MAINS)
 		external.hdd = data;
 	else
@@ -532,7 +526,6 @@ void HDDCallback(GtkWidget *widget, int data)
 
 void DisplayCallback(GtkWidget *widget, int data)
 {
-	SetSetting(SCI_DISPLAY_AUTO, data, ERROR_GUI);
 	if (source==SCI_MAINS)
 		external.display = data;
 	else
@@ -544,10 +537,14 @@ void DisplayCallback(GtkWidget *widget, int data)
 
 void AutoOffCallback(GtkWidget *widget, int data)
 {
-	if (machine.powerup==SCI_BOOT)
+	SMMRegisters reg;
+
+	reg.ebx = SCI_POWER_UP;
+ 	if (GetSetting(&reg, ERROR_GUI)==SCI_SUCCESS) {
+		if ((reg.ecx & 0xffff)==SCI_BOOT)
 			ResumeDependency();
+	}
 
-	SetSetting(SCI_SYSTEM_AUTO, data, ERROR_GUI);
 	if (source==SCI_MAINS)
 		external.autooff = data;
 	else
@@ -559,7 +556,6 @@ void AutoOffCallback(GtkWidget *widget, int data)
 
 void SpeedCallback(GtkWidget *widget, int data)
 {
-	SetSetting(SCI_PROCESSING, data, ERROR_GUI);
 	if (source==SCI_MAINS)
 		external.speed = data;
 	else
@@ -571,7 +567,6 @@ void SpeedCallback(GtkWidget *widget, int data)
 
 void SleepCallback(GtkWidget *widget, int data)
 {
-	SetSetting(SCI_SLEEP_MODE, data, ERROR_GUI);
 	if (source==SCI_MAINS)
 		external.sleep = data;
 	else
@@ -583,24 +578,17 @@ void SleepCallback(GtkWidget *widget, int data)
 
 void LCDCallback(GtkWidget *widget, int data)
 {
-	if (libretto)
-		SetSetting(SCI_LCD_BACKLIGHT, data, ERROR_GUI);
-	else
-		SetSetting(SCI_LCD_BRIGHTNESS, data, ERROR_GUI);
-
 	if (source==SCI_MAINS)
 		external.lcd = data;
 	else
 		internal.lcd = data;
 
-
 	return;
 }
 
 
 void CoolingCallback(GtkWidget *widget, int data)
 {
-	SetSetting(SCI_COOLING_METHOD, data, ERROR_GUI);
 	if (source==SCI_MAINS)
 		external.cooling = data;
 	else
@@ -612,7 +600,6 @@ void CoolingCallback(GtkWidget *widget, int data)
 
 void StandbyCallback(GtkWidget *widget, int data)
 {
-	SetSetting(SCI_STANDBY_TIME, data, ERROR_GUI);
 	if (source==SCI_MAINS)
 		external.standby = data;
 	else
@@ -640,7 +627,7 @@ GtkWidget* BuildMenu(MenuItem *list)
 				GTK_SIGNAL_FUNC(list->callback), list->data);
 		}
 
-		/* a bug in Gtk keeps gtk_widget_show_all(window) from handeling
+		/* a bug in Gtk keeps gtk_widget_show_all(window) from handling
 		   these labels properly, so show them manually for now */
 
 		gtk_widget_show(item);
@@ -653,44 +640,46 @@ GtkWidget* BuildMenu(MenuItem *list)
 
 
 /*
- * Set the settings selectable on the Battery options for the given
- * power source from the configuration file 
+ * Update the displayed sound settings so they reflect their actual settings
  */
-void SetBatterySettings(int powersource)
+void SetSoundOptions(void)
 {
-	unsigned short setting;
-	char *power;
-	
-	if (powersource==SCI_MAINS)
-		power = "ExtPowerSettings";
-	else
-		power = "BatPowerSettings";
+	SMMRegisters reg;
 
-	/* get the settings from the configuration file */
+	reg.ebx = SCI_PANEL_ALARM;	
+ 	if (GetSetting(&reg, ERROR_GUI)==SCI_SUCCESS) {
+		if ((reg.ecx & 0xffff)==SCI_ENABLED)
+			gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(panel),
+				TRUE);
+		else
+			gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(panel),
+				FALSE);
+	}
 
-	setting = GetConfigInt(power, "BatterySaveMode", 2, config);
-	SetSetting(SCI_BATTERY_SAVE, setting, ERROR_GUI);
+	reg.ebx = SCI_BATTERY_ALARM;
+	if (GetSetting(&reg, ERROR_GUI)==SCI_SUCCESS) {
+		if ((reg.ecx & 0xffff)==SCI_ENABLED)
+			gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(low),
+				TRUE);
+		else
+			gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(low),
+				FALSE);
+	}
 
-	if (setting==SCI_USER_SETTINGS) {
-		setting = GetConfigInt(power, "SleepMode", 0, config);
-		SetSetting(SCI_SLEEP_MODE, setting, ERROR_GUI);
-		setting = GetConfigInt(power, "CPUSpeed", 1, config);
-		SetSetting(SCI_PROCESSING, setting, ERROR_GUI);
-		setting = GetConfigInt(power, "SystemAutoOff", 1, config);
-		SetSetting(SCI_SYSTEM_AUTO, setting, ERROR_GUI);
+	reg.ebx = SCI_SYSTEM_BEEP;
+	if (GetSetting(&reg, ERROR_GUI)==SCI_SUCCESS) {
+		if ((reg.ecx & 0xffff)==SCI_ENABLED)
+			gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(beep),
+				TRUE);
+		else
+			gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(beep),
+				FALSE);
+	}
 
-		setting = GetConfigInt(power, "LCDBrightness", 0, config);
-		if (libretto)
-			SetSetting(SCI_LCD_BACKLIGHT, setting, ERROR_GUI);
-		else {
-			SetSetting(SCI_LCD_BRIGHTNESS, setting, ERROR_GUI);
-			setting = GetConfigInt(power, "DisplayAutoOff", 1, config);
-			SetSetting(SCI_DISPLAY_AUTO, setting, ERROR_GUI);
-			setting = GetConfigInt(power, "HDDAutoOff", 512, config);
-			SetSetting(SCI_HDD_AUTO_OFF, setting, ERROR_GUI);
-			setting = GetConfigInt(power, "CoolingMethod", 0, config);
-			SetSetting(SCI_COOLING_METHOD, setting, ERROR_GUI);
-		}
+	reg.ebx = SCI_SPEAKER_VOLUME;
+	if (GetSetting(&reg, ERROR_GUI)==SCI_SUCCESS) {
+		gtk_adjustment_set_value(GTK_ADJUSTMENT(slider),
+			(gfloat) (reg.ecx & 0xffff)+1);
 	}
 
 	return;
@@ -698,47 +687,60 @@ void SetBatterySettings(int powersource)
 
 
 /*
- * Update the displayed settings so they reflect their actual settings
+ * Update the displayed system settings so they reflect their actual settings
  */
-void SetSoundOptions(void)
+void SetSystemOptions(void)
 {
-	SciRegisters reg;
+	SMMRegisters reg;
 
-	reg.setting = SCI_PANEL_ALARM;	
+	reg.ebx = SCI_ALARM_POWER;	
  	if (GetSetting(&reg, ERROR_GUI)==SCI_SUCCESS) {
-		if (reg.current==SCI_ENABLED)
-			gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(panel),
+		if ((reg.ecx & 0xffff)==SCI_ALARM_DISABLED) {
+			gtk_widget_set_sensitive(GTK_WIDGET(hourplus), FALSE);
+			gtk_widget_set_sensitive(GTK_WIDGET(hourminus), FALSE);
+			gtk_widget_set_sensitive(GTK_WIDGET(minuteplus), FALSE);
+			gtk_widget_set_sensitive(GTK_WIDGET(minuteminus),
+				FALSE);
+			gtk_widget_set_sensitive(GTK_WIDGET(alarmam), FALSE);
+			gtk_widget_set_sensitive(GTK_WIDGET(alarmpm), FALSE);
+			gtk_widget_set_sensitive(GTK_WIDGET(alarm24), FALSE);
+			gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(disable),
+				TRUE);
+		} else {
+			gtk_widget_set_sensitive(GTK_WIDGET(hourplus), TRUE);
+			gtk_widget_set_sensitive(GTK_WIDGET(hourminus), TRUE);
+			gtk_widget_set_sensitive(GTK_WIDGET(minuteplus), TRUE);
+			gtk_widget_set_sensitive(GTK_WIDGET(minuteminus), TRUE);
+			gtk_widget_set_sensitive(GTK_WIDGET(alarmam), TRUE);
+			gtk_widget_set_sensitive(GTK_WIDGET(alarmpm), TRUE);
+			gtk_widget_set_sensitive(GTK_WIDGET(alarm24), TRUE);
+			gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(disable),
+				FALSE);
+		}
+	}
+
+	reg.ebx = SCI_POWER_UP;
+	if (GetSetting(&reg, ERROR_GUI)==SCI_SUCCESS) {
+		if ((reg.ecx & 0xffff)==SCI_RESUME)
+			gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(resume),
 				TRUE);
 		else
-			gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(panel),
+			gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(resume),
 				FALSE);
 	}
 
-	reg.setting = SCI_BATTERY_ALARM;
+	reg.ebx = SCI_PANEL_POWER;
 	if (GetSetting(&reg, ERROR_GUI)==SCI_SUCCESS) {
-		if (reg.current==SCI_ENABLED)
-			gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(low),
+		if ((reg.ecx & 0xffff)==SCI_ENABLED)
+			gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(power),
 				TRUE);
 		else
-			gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(low),
+			gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(power),
 				FALSE);
 	}
+	if (GTK_TOGGLE_BUTTON(resume)->active==FALSE)
+		gtk_widget_set_sensitive(GTK_WIDGET(power), FALSE);
 
-	reg.setting = SCI_SYSTEM_BEEP;
-	if (GetSetting(&reg, ERROR_GUI)==SCI_SUCCESS) {
-		if (reg.current==SCI_ENABLED)
-			gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(beep),
-				TRUE);
-		else
-			gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(beep),
-				FALSE);
-	}
-
-	reg.setting = SCI_SPEAKER_VOLUME;
-	if (GetSetting(&reg, ERROR_GUI)==SCI_SUCCESS) {
-		gtk_adjustment_set_value(GTK_ADJUSTMENT(slider),
-			(gfloat) reg.current+1);
-	}
 
 	return;
 }
@@ -746,14 +748,12 @@ void SetSoundOptions(void)
 
 /*
  * Save the selected settings to the configuration file, and set them
- * appropriately for the current power source when cloasing the Machine
+ * appropriately for the current power source when closing the Machine
  * Settings dialog
  */
 void MachineOkCallback(GtkWidget *widget, GtkWidget *dialog)
 {
 	char buffer[16];
-	unsigned short powersource;
-
 
 	/* delete the dialog first, makes program feel more responsive */
 
@@ -779,11 +779,6 @@ void MachineOkCallback(GtkWidget *widget, GtkWidget *dialog)
 	WriteConfigString("BatPowerSettings", "SystemAutoOff", buffer, config);
 	sprintf(buffer, "%d", internal.cooling);
 	WriteConfigString("BatPowerSettings", "CoolingMethod", buffer, config);
-	if (libretto) {
-		sprintf(buffer, "%d", internal.standby);
-		WriteConfigString("BatPowerSettings", "StandbyTime", buffer,
-			config);
-	}
 
 	/* save the mains power source settings */
 
@@ -804,16 +799,6 @@ void MachineOkCallback(GtkWidget *widget, GtkWidget *dialog)
 	WriteConfigString("ExtPowerSettings", "SystemAutoOff", buffer, config);
 	sprintf(buffer, "%d", external.cooling);
 	WriteConfigString("ExtPowerSettings", "CoolingMethod", buffer, config);
-	if (libretto) {
-		sprintf(buffer, "%d", external.standby);
-		WriteConfigString("ExtPowerSettings", "StandbyTime", buffer,
-			config);
-	}
-
-	/* set the battery options for the current power source */
-
-	powersource = SciACPower();
-	SetBatterySettings(powersource);	
 
 	/* delete the pixmaps */
 
@@ -825,6 +810,10 @@ void MachineOkCallback(GtkWidget *widget, GtkWidget *dialog)
 	gdk_bitmap_unref(plusmask);
 	gdk_bitmap_unref(minusmask);
 
+	/* exit deleting pid file */
+
+	ExitTuxTimeConfig(200);
+
 	return;
 }
 
@@ -834,8 +823,6 @@ void MachineOkCallback(GtkWidget *widget, GtkWidget *dialog)
  */
 void MachineCancelCallback(GtkWidget *widget, GtkWidget *dialog)
 {
-	unsigned short powersource;
-
 	gtk_widget_destroy(dialog);
 	dialog = NULL;
 
@@ -849,11 +836,6 @@ void MachineCancelCallback(GtkWidget *widget, GtkWidget *dialog)
 	gdk_bitmap_unref(plusmask);
 	gdk_bitmap_unref(minusmask);
 
-	/* set the battery options for the current power source */
-
-	powersource = SciACPower();
-	SetBatterySettings(powersource);	
-
 	/* return the sound settings to their entry values */
 
 	SetSetting(SCI_PANEL_ALARM, machine.panelalarm, ERROR_GUI);
@@ -866,9 +848,13 @@ void MachineCancelCallback(GtkWidget *widget, GtkWidget *dialog)
 	SetSetting(SCI_POWER_UP, machine.powerup, ERROR_GUI);
 	SetSetting(SCI_PANEL_POWER, machine.panelpower, ERROR_GUI);
 
-	printf("FIXME: need to alarm power to it's entry value "
+	printf("FIXME: need to set alarm power to it's entry value "
 		"on canceling the machine settings dialog\n");
 
+	/* exit deleting pid file */
+
+	ExitTuxTimeConfig(0);
+
 	return;
 }
 
@@ -901,7 +887,13 @@ void MachineDefaultsCallback(GtkWidget *widget, gpointer *data)
 	external.cooling = SCI_PERFORMANCE;
 	external.standby = 0;
 
+	machine.volume = SCI_VOLUME_HIGH;
+	machine.panelalarm = SCI_ENABLED;
+	machine.lowalarm = SCI_ENABLED;
+	machine.beep = SCI_ENABLED;
 	machine.alarm = SCI_ALARM_DISABLED;
+	machine.powerup = SCI_BOOT;
+	machine.panelpower = SCI_DISABLED;
 
 	/* set the default values for the sound and system pages*/
 
@@ -917,8 +909,7 @@ void MachineDefaultsCallback(GtkWidget *widget, gpointer *data)
 
 	SetBatteryOptions(powermode);
 	SetSoundOptions();
-
-	printf("FIXME: need to return System Settings to default values\n");
+	SetSystemOptions();
 
 	return;
 }
@@ -929,7 +920,7 @@ void MachineDefaultsCallback(GtkWidget *widget, gpointer *data)
  */
 void AddBatteryPage(GtkWidget *notebook, GtkWidget *window)
 {
-	SciRegisters reg;
+	SMMRegisters reg;
 	int placement;
 	GtkWidget *box,*frame,*buttons,*options;
 	GtkWidget *label,*operation,*button,*table;
@@ -975,15 +966,22 @@ void AddBatteryPage(GtkWidget *notebook, GtkWidget *window)
 	/* determine the battery save mode and LCD types for the machine.
 	   need info on machine ID's for Atlanta and Derby machines */
 
-	reg.setting = SCI_BATTERY_SAVE;
+	reg.ebx = SCI_BATTERY_SAVE;
 	GetSetting(&reg, ERROR_GUI);
-	if (reg.possible==3)
+	if ((reg.edx & 0xffff)==3)
 		savetype = STOCKHOLM;
 	else
 		savetype = MAUI;
-	reg.setting = SCI_LCD_BRIGHTNESS;
-	GetSetting(&reg, ERROR_GUI);
 
+	lcdtype = BACKLIGHT;
+	reg.ebx = SCI_LCD_BRIGHTNESS;
+	if (GetSetting(&reg, ERROR_GUI)==SCI_SUCCESS) {
+		if ((reg.ecx & 0xffff)==0x8001)
+			lcdtype = SUPER;
+		if ((reg.ecx & 0xffff)==0x0000)
+			lcdtype = BRIGHT;
+	}
+	
 	/* now create the widgets for the page */
 
 	box = gtk_hbox_new(FALSE, 4);
@@ -1066,15 +1064,6 @@ void AddBatteryPage(GtkWidget *notebook, GtkWidget *window)
 		GTK_SHRINK | GTK_FILL | GTK_EXPAND, 0, 2);
 	placement++;
 
-	if (libretto) {
-		label = gtk_label_new("Standby Time");
-		gtk_misc_set_alignment(GTK_MISC(label), 0.0, 1.0);
-		gtk_table_attach(GTK_TABLE(table), label, 0, 1, placement,
-			placement+1, GTK_SHRINK | GTK_FILL | GTK_EXPAND,
-			GTK_SHRINK | GTK_FILL | GTK_EXPAND,0, 2);
-		placement++;
-	}
-
 	label = gtk_label_new("Processing Speed");
 	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 1.0);
 	gtk_table_attach (GTK_TABLE(table), label, 0, 1, placement, placement+1,
@@ -1089,24 +1078,19 @@ void AddBatteryPage(GtkWidget *notebook, GtkWidget *window)
 		GTK_SHRINK | GTK_FILL | GTK_EXPAND, 0, 2);
 	placement++;
 
-	if (libretto)
-		label = gtk_label_new("LCD Backlight");
-	else
-		label = gtk_label_new("LCD Brightness");
+	label = gtk_label_new("LCD Brightness");
 	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 1.0);
 	gtk_table_attach (GTK_TABLE(table), label, 0, 1, placement, placement+1,
 		GTK_SHRINK | GTK_FILL | GTK_EXPAND,
 		GTK_SHRINK | GTK_FILL | GTK_EXPAND, 0, 2);
 	placement++;
 
-	if (!libretto) {
-		label = gtk_label_new("Cooling Method");
-		gtk_misc_set_alignment(GTK_MISC(label), 0.0, 1.0);
-		gtk_table_attach(GTK_TABLE(table), label, 0, 1, placement,
-			placement+1, GTK_SHRINK | GTK_FILL | GTK_EXPAND,
-			GTK_SHRINK | GTK_FILL | GTK_EXPAND, 0, 2);
-		placement++;
-	}
+	label = gtk_label_new("Cooling Method");
+	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 1.0);
+	gtk_table_attach(GTK_TABLE(table), label, 0, 1, placement,
+		placement+1, GTK_SHRINK | GTK_FILL | GTK_EXPAND,
+		GTK_SHRINK | GTK_FILL | GTK_EXPAND, 0, 2);
+	placement++;
 
 	/* now the option buttons */
 
@@ -1136,8 +1120,6 @@ void AddBatteryPage(GtkWidget *notebook, GtkWidget *window)
 			break;
 	}
 
-printf("SAVE TYPE = %d %d\n",savetype, MAUI);
-
 	menu = BuildMenu(hddmenu);
 	hdd = gtk_option_menu_new();
 	gtk_table_attach(GTK_TABLE(table), hdd, 1, 2, placement, placement+1,
@@ -1168,18 +1150,6 @@ printf("SAVE TYPE = %d %d\n",savetype, MAUI);
 	if (powermode!=SCI_USER_SETTINGS)
 		gtk_widget_set_sensitive(GTK_WIDGET(autooff), FALSE);
 
-	if (libretto) {
-		menu = BuildMenu(standbymenu);
-		standby = gtk_option_menu_new();
-		gtk_table_attach(GTK_TABLE(table), standby, 1, 2, placement,
-			placement+1, GTK_SHRINK | GTK_FILL | GTK_EXPAND,
-			GTK_SHRINK | GTK_FILL | GTK_EXPAND, 4, 2);
-		gtk_option_menu_set_menu(GTK_OPTION_MENU(standby), menu);
-		placement++;
-		if (powermode!=SCI_USER_SETTINGS)
-			gtk_widget_set_sensitive(GTK_WIDGET(standby), FALSE);
-	}
-
 	menu = BuildMenu(speedmenu);
 	speed = gtk_option_menu_new();
 	gtk_table_attach(GTK_TABLE(table), speed, 1, 2, placement, placement+1,
@@ -1200,10 +1170,13 @@ printf("SAVE TYPE = %d %d\n",savetype, MAUI);
 	if (powermode!=SCI_USER_SETTINGS)
 		gtk_widget_set_sensitive(GTK_WIDGET(sleep), FALSE);
 
-	if (libretto)
-		menu = BuildMenu(lcdbackmenu);
-	else
+	if (lcdtype==BRIGHT)
 		menu = BuildMenu(lcdbrightmenu);
+	else if (lcdtype==SUPER)
+		menu = BuildMenu(lcdsupermenu);
+	else
+		menu = BuildMenu(lcdbackmenu);
+
 	lcd = gtk_option_menu_new();
 	gtk_table_attach(GTK_TABLE(table), lcd, 1, 2, placement, placement+1,
 		GTK_SHRINK | GTK_FILL | GTK_EXPAND,
@@ -1213,17 +1186,15 @@ printf("SAVE TYPE = %d %d\n",savetype, MAUI);
 	if (powermode!=SCI_USER_SETTINGS)
 		gtk_widget_set_sensitive(GTK_WIDGET(lcd), FALSE);
 
-	if (!libretto) {
-		menu = BuildMenu(coolingmenu);
-		cooling = gtk_option_menu_new();
-		gtk_table_attach(GTK_TABLE(table), cooling, 1, 2, placement,
-			placement+1, GTK_SHRINK | GTK_FILL | GTK_EXPAND,
-			GTK_SHRINK | GTK_FILL | GTK_EXPAND, 4, 2);
-		gtk_option_menu_set_menu(GTK_OPTION_MENU(cooling), menu);
-		placement++;
-		if (powermode!=SCI_USER_SETTINGS)
-			gtk_widget_set_sensitive(GTK_WIDGET(cooling), FALSE);
-	}
+	menu = BuildMenu(coolingmenu);
+	cooling = gtk_option_menu_new();
+	gtk_table_attach(GTK_TABLE(table), cooling, 1, 2, placement,
+		placement+1, GTK_SHRINK | GTK_FILL | GTK_EXPAND,
+		GTK_SHRINK | GTK_FILL | GTK_EXPAND, 4, 2);
+	gtk_option_menu_set_menu(GTK_OPTION_MENU(cooling), menu);
+	placement++;
+	if (powermode!=SCI_USER_SETTINGS)
+		gtk_widget_set_sensitive(GTK_WIDGET(cooling), FALSE);
 
 	/* set the option menus to match the current battery save mode */
 
@@ -1340,34 +1311,34 @@ void VolumeUpdate(GtkAdjustment *adjustment, int *junk)
  */
 void AddSoundPage(GtkWidget *notebook, GtkWidget *window)
 {
-	SciRegisters reg;
+	SMMRegisters reg;
 	GtkWidget *frame,*sound,*speaker;
 	GtkWidget *box,*buttons;
 	GtkWidget *label,*volume;
 
 	/* get the sound settings from the BIOS and store for cancel */
 
-	reg.setting = SCI_SPEAKER_VOLUME;
+	reg.ebx = SCI_SPEAKER_VOLUME;
  	if (GetSetting(&reg, ERROR_GUI)==SCI_SUCCESS)
-		machine.volume = reg.current;
+		machine.volume = (reg.ecx & 0xffff);
 	else
 		machine.volume = SCI_VOLUME_HIGH;
 
-	reg.setting = SCI_PANEL_ALARM;
+	reg.ebx = SCI_PANEL_ALARM;
  	if (GetSetting(&reg, ERROR_GUI)==SCI_SUCCESS)
-		machine.panelalarm = reg.current;
+		machine.panelalarm = (reg.ecx & 0xffff);
 	else
 		machine.panelalarm = SCI_ENABLED;
 
-	reg.setting = SCI_BATTERY_ALARM;
+	reg.ebx = SCI_BATTERY_ALARM;
  	if (GetSetting(&reg, ERROR_GUI)==SCI_SUCCESS)
-		machine.lowalarm = reg.current;
+		machine.lowalarm = (reg.ecx & 0xffff);
 	else
 		machine.lowalarm = SCI_ENABLED;
 
-	reg.setting = SCI_SYSTEM_BEEP;
+	reg.ebx = SCI_SYSTEM_BEEP;
  	if (GetSetting(&reg, ERROR_GUI)==SCI_SUCCESS)
-		machine.beep = reg.current;
+		machine.beep = (reg.ecx & 0xffff);
 	else
 		machine.beep = SCI_ENABLED;
 
@@ -1601,11 +1572,32 @@ void AddSystemPage(GtkWidget *notebook, GtkWidget *window)
 	GtkWidget *box,*frame,*label;
 	GtkWidget *time,*format;
 	GtkWidget *buttons,*system,*alarm,*timebox;
-	GtkWidget *disable,*resume;
 	GSList *radio;
-	SciRegisters reg;
+	SMMRegisters reg;
 	char buffer[16];
 
+	/* get the system settings from the BIOS and store for cancel */
+
+	reg.ebx = SCI_ALARM_POWER;
+ 	if (GetSetting(&reg, ERROR_GUI)==SCI_SUCCESS)
+		machine.alarm = (reg.ecx & 0xffff);
+	else
+		machine.alarm = SCI_ALARM_DISABLED;
+
+	reg.ebx = SCI_POWER_UP;
+ 	if (GetSetting(&reg, ERROR_GUI)==SCI_SUCCESS)
+		machine.powerup = (reg.ecx & 0xffff);
+	else
+		machine.powerup = SCI_BOOT;
+
+	reg.ebx = SCI_PANEL_POWER;
+ 	if (GetSetting(&reg, ERROR_GUI)==SCI_SUCCESS)
+		machine.panelpower = (reg.ecx & 0xffff);
+	else
+		machine.panelpower = SCI_DISABLED;
+
+	/* now setup the notebook page */
+
 	box = gtk_hbox_new(FALSE, 0);
 
 	frame = gtk_frame_new("System Configuration");
@@ -1647,13 +1639,10 @@ void AddSystemPage(GtkWidget *notebook, GtkWidget *window)
 	gtk_container_border_width(GTK_CONTAINER(format), 4);
 	gtk_container_add(GTK_CONTAINER(frame), format);
 
-
 	/* Get the alarm time and set the label */
 
 	GetConfigString("TuxTime", "AlarmPowerOn", "12:00H", buffer, 16, config);
-	reg.setting = SCI_ALARM_POWER;
- 	GetSetting(&reg, ERROR_GUI);
-	alarmon = SCI_TIME_ON(reg.current);
+	alarmon = SCI_TIME_ON(machine.alarm);
 	if (ConvertTime(buffer, &hours, &minutes)==0) {
 		hours = 12;
 		minutes = 0;
@@ -1762,9 +1751,7 @@ void AddSystemPage(GtkWidget *notebook, GtkWidget *window)
 
 	resume = gtk_check_button_new_with_label("Resume Mode");
 	gtk_box_pack_start(GTK_BOX(system), resume, FALSE, FALSE, 2);
-	reg.setting = SCI_POWER_UP;
-	GetSetting(&reg, ERROR_GUI);
-	if (reg.current==SCI_RESUME)
+	if (machine.powerup==SCI_RESUME)
 		gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(resume), TRUE);
 	gtk_signal_connect(GTK_OBJECT(resume), "toggled",
 		GTK_SIGNAL_FUNC(ResumeCallback), NULL);
@@ -1772,9 +1759,7 @@ void AddSystemPage(GtkWidget *notebook, GtkWidget *window)
 	power = gtk_check_button_new_with_label("Panel Power On/Off");
 	gtk_box_pack_start(GTK_BOX(system), power, FALSE, FALSE, 2);
 
-	reg.setting = SCI_PANEL_POWER;
-	GetSetting(&reg, ERROR_GUI);
-	if (reg.current==SCI_ENABLED)
+	if (machine.panelpower==SCI_ENABLED)
 		gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(power), TRUE);
 	if (GTK_TOGGLE_BUTTON(resume)->active==FALSE)
 		gtk_widget_set_sensitive(GTK_WIDGET(power), FALSE);
@@ -1787,69 +1772,51 @@ void AddSystemPage(GtkWidget *notebook, GtkWidget *window)
 	return;
 }
 
+
 /*
  * Create the Machine Settings dialog
  */
-void MachineDialog(void)
+void MachineDialog(GtkWidget *window)
 {
-	static GtkWidget *window = NULL;
 	GtkWidget *notebook;
 	GdkColor *transparent = NULL;
 
-	if (!window) {
-		window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+	/* create a new notebook, place the position of the tabs */
 
-		gtk_signal_connect(GTK_OBJECT(window), "destroy",
-			GTK_SIGNAL_FUNC(gtk_widget_destroyed), &window);
+	notebook = gtk_notebook_new();
+	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(notebook), GTK_POS_TOP);
+	gtk_container_add(GTK_CONTAINER(window), notebook);
 
-		gtk_window_set_title(GTK_WINDOW(window), "Machine Settings");
-		gtk_container_border_width(GTK_CONTAINER(window), 6);
+	/* create the pixmaps used by the dialog */
 
-		/* create a new notebook, place the position of the tabs */
+	plus = gdk_pixmap_create_from_xpm_d(window->window,
+		&plusmask, transparent, plus_xpm);
+	minus = gdk_pixmap_create_from_xpm_d(window->window,
+		&minusmask, transparent, minus_xpm);
+	plusdisabled = gdk_pixmap_create_from_xpm_d(window->window,
+		&plusmask, transparent, plusdisabled_xpm);
+	minusdisabled = gdk_pixmap_create_from_xpm_d(window->window,
+		&minusmask, transparent, minusdisabled_xpm);
+	chargedpixmap = gdk_pixmap_create_from_xpm_d(window->window,
+		&chargedmask, transparent, batterycharged_xpm);
+	hourpluspix = gtk_pixmap_new(plus, plusmask);
+	hourplusdisabledpix = gtk_pixmap_new(plusdisabled, plusmask);
+	hourminuspix = gtk_pixmap_new(minus, minusmask);
+	hourminusdisabledpix = gtk_pixmap_new(minusdisabled, minusmask);
+	minutepluspix = gtk_pixmap_new(plus, plusmask);
+	minuteplusdisabledpix = gtk_pixmap_new(plusdisabled, plusmask);
+	minuteminuspix = gtk_pixmap_new(minus, minusmask);
+	minuteminusdisabledpix = gtk_pixmap_new(minusdisabled, minusmask);
 
-		notebook = gtk_notebook_new();
-		gtk_notebook_set_tab_pos(GTK_NOTEBOOK(notebook), GTK_POS_TOP);
-		gtk_container_add(GTK_CONTAINER(window), notebook);
-		gtk_widget_realize(notebook);
+	/* add the three pages to the notebook */
 
-		/* create the pixmaps used by the dialog */
+	AddBatteryPage(notebook, window);
+	AddSoundPage(notebook, window);
+	AddSystemPage(notebook, window);
 
-		plus = gdk_pixmap_create_from_xpm_d(notebook->window,
-			&plusmask, transparent, plus_xpm);
-		minus = gdk_pixmap_create_from_xpm_d(notebook->window,
-			&minusmask, transparent, minus_xpm);
-		plusdisabled = gdk_pixmap_create_from_xpm_d(notebook->window,
-			&plusmask, transparent, plusdisabled_xpm);
-		minusdisabled = gdk_pixmap_create_from_xpm_d(notebook->window,
-			&minusmask, transparent, minusdisabled_xpm);
-		chargedpixmap = gdk_pixmap_create_from_xpm_d(notebook->window,
-			&chargedmask, transparent, batterycharged_xpm);
+	/* set to start at first page */
 
-		hourpluspix = gtk_pixmap_new(plus, plusmask);
-		hourplusdisabledpix = gtk_pixmap_new(plusdisabled, plusmask);
-		hourminuspix = gtk_pixmap_new(minus, minusmask);
-		hourminusdisabledpix = gtk_pixmap_new(minusdisabled, minusmask);
-		minutepluspix = gtk_pixmap_new(plus, plusmask);
-		minuteplusdisabledpix = gtk_pixmap_new(plusdisabled, plusmask);
-		minuteminuspix = gtk_pixmap_new(minus, minusmask);
-		minuteminusdisabledpix = gtk_pixmap_new(minusdisabled, minusmask);
-
-
-		/* add the three pages to the notebook */
-
-		AddBatteryPage(notebook, window);
-		AddSoundPage(notebook, window);
-		AddSystemPage(notebook, window);
-
-		/* set to start at first page */
-
-		gtk_notebook_set_page(GTK_NOTEBOOK(notebook), 0);
-	}
-
-	if (!GTK_WIDGET_VISIBLE(window))
-		gtk_widget_show_all(window);
-	else
-		gtk_widget_destroy(window);
+	gtk_notebook_set_page(GTK_NOTEBOOK(notebook), 0);
 
 	return;
 }
