@@ -11,6 +11,12 @@
  *   not to make any modifications and use it as it stands.
  *
  * $Log: sci.h,v $
+ * Revision 1.10  1999/12/18 14:33:44  jab
+ * removed prototype for SciGetModel
+ *
+ * Revision 1.9  1999/12/04 13:41:23  jab
+ * modified SCI_DATE macro to exclude the year and added SCI_FULLDATE macro
+ *
  * Revision 1.8  1999/03/11 20:23:18  jab
  * added macros to manipulate date type
  * updated some of the enumerations and added a few more
@@ -40,19 +46,19 @@
  * Initial revision
  *
  *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 675 Mass Ave, Cambridge, MA 02139, USA.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software 
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  */
 
@@ -66,23 +72,15 @@ extern "C" {
 #ifndef SMMREGISTERS
 #define SMMREGISTERS
 typedef struct {
-	unsigned short ax;
-	unsigned short bx;
-	unsigned short cx;
-	unsigned int edx;
-	unsigned int esi;
-	unsigned short di;
+	unsigned long eax;
+	unsigned long ebx;
+	unsigned long ecx;
+	unsigned long edx;
+	unsigned long esi;
+	unsigned long edi;
 } SMMRegisters;
 #endif
 
-typedef struct {
-	unsigned short setting;
-	unsigned short current;
-	unsigned short possible;
-	unsigned short defaults;
-	unsigned char attribute;
-} SciRegisters;
-
 
 /*
  * the different modes that can be changed
@@ -272,7 +270,8 @@ enum {
 #define SCI_HOUR(t)   (t & 0x7fc0)>>7 
 #define SCI_MINUTE(t) (t & 0x7e)>>1
 #define SCI_TIME_ON(t) (t & 0x01)
-#define SCI_DATE(y,m,d) (((y-1990) & 0x1f)<<10) | ((m & 0xf)<<6) | ((d & 0x1f)<<1)
+#define SCI_DATE(m,d) ((m & 0xf)<<6) | ((d & 0x1f)<<1)
+#define SCI_FULLDATE(y,m,d) (((y-1990) & 0x1f)<<10) | ((m & 0xf)<<6) | ((d & 0x1f)<<1)
 #define SCI_YEAR(d) 1990+((d & 0x1fc00)>>10)
 #define SCI_MONTH(d) (d & 0x3c0)>>6
 #define SCI_DAY(d) (d & 0x3e)>>1
@@ -284,10 +283,9 @@ enum {
 int SciSupportCheck(int *version);
 int SciOpenInterface(void);
 int SciCloseInterface(void);
-int SciGet(SciRegisters *reg);
-int SciSet(SciRegisters *reg);
+int SciGet(SMMRegisters *reg);
+int SciSet(SMMRegisters *reg);
 int SciACPower(void);
-int SciGetModel(char *model);
 int SciSetPassword(char *password, unsigned short type, int *trys);
 
 #ifdef __cplusplus
