@@ -2,7 +2,21 @@
  *
  * Copyright (c) 1997-99  Jonathan A. Buzzard (jonathan@buzzard.org.uk)
  *
- * $Log:$
+ * Based on
+ *       wmapm by Chris D. Faulhaber <jedgar@speck.ml.org> and
+ *                Michael G. Henderson <mghenderson@lanl.gov>
+ *       mount.app by Steve Borho <borho@stlnet.com>
+ *
+ * $Log: wmtuxtime.c,v $
+ * Revision 1.3  1999/12/17 12:32:28  jab
+ * added lots of machine ID's to stop emails with already known models
+ *
+ * Revision 1.2  1999/12/04 13:42:38  jab
+ * added titles to battery warning dialog boxes
+ *
+ * Revision 1.1  1999/12/01 19:55:28  jab
+ * Initial revision
+ *
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,14 +34,16 @@
  *
  */
 
-static char const rcsid[]="$Id:$";
+static char const rcsid[]="$Id: wmtuxtime.c,v 1.3 1999/12/17 12:32:28 jab Exp jab $";
 
 
 #include<stdio.h>
 #include<stdlib.h>
 #include<string.h>
 #include<unistd.h>
+#include<fcntl.h>
 #include<sys/stat.h>
+#include<sys/wait.h>
 #include<errno.h>
 #include<signal.h>
 #include<paths.h>
@@ -51,15 +67,18 @@ static char const rcsid[]="$Id:$";
 
 enum { EMPTY, PRIMARY, AUXILLARY };
 
-int id;
-int power = SCI_MAINS;
-int selectbay = EMPTY;
-int alarms;
-char *config;
+static int id;
+static int power = SCI_MAINS;
+static int selectbay = EMPTY;
+static int alarms;
+static char *config;
 static BatteryState low,critical;
 
-
-#define COMMAND "echo -e \'%s\' | xmessage -buttons Ok -center -file -"
+#define PMAX 5
+static long pid[PMAX];
+static int retstatus[PMAX];
+static int phead = 0;
+static int ptail = 0;
 
 
 /*
@@ -75,6 +94,35 @@ void CatchSignal(int x)
 }
 
 
+/*
+ * Handle SIGCHLD signals
+ *
+ * This function is called by the kernel when one of our child processes dies.
+ * We need to call wait to get the status of the child and save it in a ring
+ * buffer so it can be examined later. Before returning, we re-register the
+ * signal handler.
+ */
+void NoZombie(int sig)
+{
+	long exited = 0;
+	int status = 0;
+
+	exited = wait(&status);
+
+	if ((phead==ptail-1) || (phead==PMAX-1 && ptail==0)) {
+		/* Ahhhhhhh. Fork bomb */
+		return;
+	} else {
+		pid[phead] = exited;
+		retstatus[phead] = status;
+		phead = (phead==PMAX-1) ? 0 : ++phead;
+	}
+
+	signal(SIGCHLD, NoZombie);
+	return;
+}
+
+
 /*
  * Install signal handlers so we can clean up nicely on an error
  */
@@ -90,27 +138,88 @@ void HandleSignals(void)
 	signal(SIGFPE, CatchSignal);
 	signal(SIGKILL, CatchSignal);
 	signal(SIGSEGV, CatchSignal);
-	signal(SIGPIPE, CatchSignal);
+	signal(SIGPIPE, SIG_IGN);
 	signal(SIGTERM, CatchSignal);
 	signal(SIGSTOP, CatchSignal);
 	signal(SIGTTIN, CatchSignal);
 	signal(SIGTTOU, CatchSignal);
+	signal(SIGCHLD, NoZombie);
 	signal(SIGUSR1, SIG_IGN);
 
 	return;
 }
 
 
+/*
+ * Use xmessage to display the battery alert messages
+ */
+int DisplayMessage(char *message, char *title)
+{
+	int pid,save,lead[2];
+
+	/* create the pipe */
+
+	if (pipe(lead)<0) {
+		fprintf(stderr, "wmtuxtime: unable to create pipe to "
+			"xmessage\n");
+		return -1;
+	}
+
+	if (fcntl(lead[0], F_SETFL, O_NDELAY)<0)
+		fprintf(stderr, "wmtuxtime: fatal fcntl call\n");
+
+	pid = fork();
+	if (pid>0) {
+		save = dup(1);
+		close(1);
+		dup(lead[1]);
+		close(lead[0]);
+		close(lead[1]);
+		fprintf(stdout, "%s\n", message);
+		close(1);
+		dup(save);
+		return 127;
+	} else if (pid==0) {
+		char *argv[9] = {"xmessage", "-buttons", "Ok", "-title", title,
+			"-center", "-file", "-", NULL};
+		close(0);
+		dup(lead[0]);
+		close(lead[0]);
+		close(lead[1]);
+		execv(XMESSAGE, argv);
+		return 127;
+	}
+
+	return -1;
+}
+
+
+/*
+ * Call the TuxTime configuration manager
+ */
+int CallConfigApp(char *settings)
+{
+	if (!fork()) {
+		char *argv[3] = {"tuxtime-conf", settings, NULL};
+
+		execv(BINDIR"/tuxtime-conf", argv);
+		return 127;
+	}
+
+	return -1;
+}
+
+
 /*
  * Trigger a software suspend sequence using the HCI
  */
 void SoftwareSuspend(void)
 {
-	HciRegisters reg;
+	SMMRegisters reg;
 
-	reg.ax = HCI_SET;
-	reg.bx = HCI_SOFTWARE_SUSPEND;
-	reg.cx = HCI_ENABLE;
+	reg.eax = HCI_SET;
+	reg.ebx = HCI_SOFTWARE_SUSPEND;
+	reg.ecx = HCI_ENABLE;
 
 	HciFunction(&reg);
 
@@ -121,10 +230,9 @@ void SoftwareSuspend(void)
 /*
  * Handle button presses
  */
-void pressEvent(XButtonEvent *event)
+void PressEvent(XButtonEvent *event)
 {	
 	int x,y;
-	char message[128],line[192];
 	
 	x = event->x;
 	y = event->y;
@@ -148,44 +256,42 @@ void pressEvent(XButtonEvent *event)
 					selectbay = PRIMARY;
 					break;
 			}
+		} else {
+			XBell(display, 100);
 		}
-		RedrawWindow();
-		printf("BUTTON: next\n");
-		usleep(20000L);
 	} else if (x>=19 && y>=48 && x<=31 && y<=58){
 		/* alarm settings */
 
 		copyXPMArea(79, 44, 13, 11, 19, 48);
-		RedrawWindow();
-		printf("BUTTON: alarms\n");
-		sprintf(message, TIME_BATTERY_ALERT, 20);
-		sprintf(line, COMMAND, message);
-		printf("%s\n", line);
-		system(line);
-
-		usleep(20000L);
+		CallConfigApp("--alarms");
 	} else if (x>=32 && y>=48 && x<=44 && y<=58){
 		/* suspend call */
 
 		copyXPMArea(92, 44, 13, 11, 32, 48);
-		RedrawWindow();
-		printf("BUTTON: suspend\n");
 		SoftwareSuspend();
-		usleep(20000L);
 	} else if (x>=45 && y>=48 && x<=57 && y<=58){
 		/* machine settings */
 
 		copyXPMArea(105, 44, 13, 11, 45, 48);
-		RedrawWindow();
-		system("/home/jab/c/toshutils/src/tuxtime");
-		printf("BUTTON: settings\n");
-		usleep(20000L);
+		CallConfigApp("--settings");
 	}
 
 	return;
 }
 
 
+/*
+ * Repaint the buttons in the non-depressed state on release
+ */
+void ReleaseEvent(XButtonEvent *event)
+{
+	copyXPMArea(66, 56, 52, 11, 6, 48);
+	RedrawWindow();
+
+	return;
+}
+
+
 /*
  * Convert a single character to an interger, similar to atoi 
  */
@@ -231,12 +337,13 @@ int ConvertTime(char *time, int *hour, int *minute)
 
 	/* deal with any am or pm switch */
 
-	if (*(time+2)!='\0')
+	if (*(time+2)!='\0') {
 		if ((*(time+2)=='p') || (*(time+2)=='P'))
 			*hour+=12;
 		else if ((*(time+2)!='h') && (*(time+2)!='H') &&
 			(*(time+2)!='a') && (*(time+2)!='A'))
 			return 0;
+	}
 
 	/* check to see if the converted time is valid */
 
@@ -350,8 +457,8 @@ void PowerChanged(int source, char *file)
 {
 	unsigned short save,hdd,display,sleep;
 	unsigned short speed,lcd,autooff,cooling;
-	SciRegisters reg;
-	int i;
+	SMMRegisters reg;
+	int user;
 	char *section;
 
 	/* read the settings from the config file */
@@ -376,19 +483,60 @@ void PowerChanged(int source, char *file)
 
 	/* try and set the battery save mode */
 
-	reg.setting = SCI_BATTERY_SAVE;
-	reg.current = save;
+	reg.ebx = SCI_BATTERY_SAVE;
+	reg.ecx = save;
 	if (SciSet(&reg)==SCI_SUCCESS) {
-		i = 0;
+		user = 0;
 	} else {
-		i = 1;
+		user = 1;
 	}
 
-	printf("SAVEMODE=%d\n", i);
-
 	/* set the other settings if user mode or new style machine */
 
-	if (save==SCI_USER_SETTINGS) {
+	if ((save==SCI_USER_SETTINGS) || (user==1)) {
+		reg.ebx = SCI_HDD_AUTO_OFF;
+		reg.ecx = hdd;
+		if (SciSet(&reg)==SCI_FAILURE) {
+			fprintf(stderr, "wmtuxtime: unable to set HDD "
+				"auto off\n");
+		}
+		reg.ebx = SCI_DISPLAY_AUTO;
+		reg.ecx = display;
+		if (SciSet(&reg)==SCI_FAILURE) {
+			fprintf(stderr, "wmtuxtime: unable to set display "
+				"auto off\n");
+		}
+		reg.ebx = SCI_SLEEP_MODE;
+		reg.ecx = sleep;
+		if (SciSet(&reg)==SCI_FAILURE) {
+			fprintf(stderr, "wmtuxtime: unable to set sleep "
+				"mode\n");
+		}
+		reg.ebx = SCI_PROCESSING;
+		reg.ecx = speed;
+		if (SciSet(&reg)==SCI_FAILURE) {
+			fprintf(stderr, "wmtuxtime: unable to set CPU "
+				"speed\n");
+		}
+		reg.ebx = SCI_LCD_BRIGHTNESS;
+		reg.ecx = lcd;
+		if (SciSet(&reg)==SCI_FAILURE) {
+			fprintf(stderr, "wmtuxtime: unable to set LCD "
+				"brightness\n");
+		}
+		reg.ebx = SCI_SYSTEM_AUTO;
+		reg.ecx = autooff;
+		if (SciSet(&reg)==SCI_FAILURE) {
+			fprintf(stderr, "wmtuxtime: unable to set system "
+				"auto off\n");
+		}
+		reg.ebx = SCI_COOLING_METHOD;
+		reg.ecx = cooling;
+		if (SciSet(&reg)==SCI_FAILURE) {
+			fprintf(stderr, "wmtuxtime: unable to set cooling "
+				"method\n");
+		}
+
 	}
 
 	SciCloseInterface();
@@ -400,17 +548,16 @@ void PowerChanged(int source, char *file)
 /*
  * Display the battery alarm messages using xmessage or if avaliable
  * wmessage
- *
  */
 void DisplayAlarm(int state, int percent, int time)
 {
 	int trigger,hour,minute;
- 	char message[128],line[192];
+ 	char message[128];
 
 	if (state==EMERGENCY) {
-		sprintf(message, EMERGENCY_BATTERY_ALERT);
-		sprintf(line, COMMAND, message);
-		system(line);
+		XBell(display, 100);
+		sprintf(message, "%s", EMERGENCY_ALERT);
+		DisplayMessage(message, "Emergency Battery Alert");
 		return;
 	}
 
@@ -418,34 +565,75 @@ void DisplayAlarm(int state, int percent, int time)
 	minute = SCI_MINUTE(time);
 	trigger = NEVER;
 
-
-	/* determine what triggered the alarm */
+	/* determine what triggered the alarm and from the correct message */
 
 	if (state==LOW)
 		trigger = BatteryTest(&low, percent, MINUTES(hour, minute));
 	else if (state==CRITICAL)
 		trigger = BatteryTest(&critical, percent, MINUTES(hour, minute));
 
-	/* display the correct alarm message */
+	/* form the correct message */
 
-	if (trigger & TIME) {
-		sprintf(message, TIME_BATTERY_ALERT, MINUTES(hour, minute));
-		sprintf(line, COMMAND, message);
-		system(line);
-	}
-
-	if (trigger & PERCENT) {
-		sprintf(message, PERCENT_BATTERY_ALERT, percent);
-		sprintf(line, COMMAND, message);
-		system(line);
-	}
+	if (trigger & TIME)
+		sprintf(message, TIME_ALERT, MINUTES(hour, minute));
+	else if (trigger & PERCENT)
+		sprintf(message, PERCENT_ALERT, percent);
 	
+	/* display the message */
 
+	if (trigger!=NEVER) {
+		XBell(display, 100);
+		if (state==LOW)
+			DisplayMessage(message, "Low Battery Alarm");
+		else if (state==CRITICAL)
+			DisplayMessage(message, "Critical Battery Alarm");
+	}
 
 	return;
 }
 
 
+/*
+ * Determine if the battery is charging by reading /proc/apm if avaliable
+ */
+int BatteryCharging(void)
+{
+	FILE *str;
+	int battery;
+	char version[10],buffer[64];
+
+
+	if (access("/proc/apm", R_OK)) {
+		/* cannot find /proc/apm */
+		fprintf(stderr, "wmtuxtime: unable to access /proc/apm\n");
+		return 0;
+	} else {
+
+		/* open /proc/apm for reading */
+		if (!(str = fopen("/proc/apm", "r"))) {
+			fprintf(stderr, "wmtuxtime: unable to open /proc/apm\n");
+			return 0;
+		}
+
+		/* scan in the information */
+		fgets(buffer, sizeof(buffer)-1, str);
+		fclose(str);
+		buffer[sizeof(buffer)-1] = '\0';
+		sscanf(buffer, "%s %*d.%*d %*x %*x %x %*x %*d%% %*d %*s\n",
+			version, &battery);
+
+		/* if old style driver return */
+		if (version[0]=='B') {
+			fprintf(stderr, "wmtuxtime: old style apm driver\n");
+			return 0;
+		}
+
+		return battery;
+	}
+
+	return 0;
+}
+
 
 /*
  * Update the battery life remaining display, and switch settings if the
@@ -453,8 +641,9 @@ void DisplayAlarm(int state, int percent, int time)
  */
 void Update(void)
 {
-	SciRegisters reg;
-	int time,percent,second,battery,hour,minute,state,i,j;
+	SMMRegisters reg;
+	int time,percent,second,battery;
+	int hour,minute,state,i,j;
 
 	time = 0;
 	percent = 0;
@@ -462,24 +651,24 @@ void Update(void)
 	/* get the percent and time remaining */
 
 	SciOpenInterface();
-	reg.setting = SCI_BATTERY_PERCENT;
+	reg.ebx = SCI_BATTERY_PERCENT;
 	if (SciGet(&reg)==SCI_SUCCESS)
-		percent = ((100*reg.current)/reg.possible);
+		percent = ((100*(reg.ecx & 0xffff))/(reg.edx & 0xffff));
 	else
 		percent = -1;
 
-	reg.setting = SCI_BATTERY_TIME;
+	reg.ebx = SCI_BATTERY_TIME;
 	if (SciGet(&reg)==SCI_SUCCESS)
-		time = reg.current;
+		time = (reg.ecx & 0xffff);
 	else
 		time = -1;
 
 	/* check to see if their is a battery in the SelectBay */
 
-	reg.setting = SCI_2ND_BATTERY;
+	reg.ebx = SCI_2ND_BATTERY;
 	if (SciGet(&reg)==SCI_SUCCESS) {
-		if (reg.current!=0xffff)
-			second = ((100*reg.current)/reg.possible);
+		if ((reg.ecx & 0xffff)!=0xffff)
+			second = ((100*(reg.ecx & 0xffff))/(reg.edx & 0xffff));
 		else
 			second = -1;
 		if ((power==SCI_BATTERY) && (selectbay==EMPTY))
@@ -512,12 +701,10 @@ void Update(void)
 		DisplayAlarm(state, percent, time);
 	}
 	if ((state==LOW) && ((alarms & LOW)==0)) {
-		fprintf(stderr,"LOW BATTERY ALARM\n");	
 		alarms |= LOW;
 		DisplayAlarm(state, percent, time);
 	}
 	if ((state==CRITICAL) && ((alarms & CRITICAL)==0)) {
-		fprintf(stderr,"LOW BATTERY ALARM\n");	
 		alarms |= CRITICAL;
 		DisplayAlarm(state, percent, time);
 	}
@@ -527,13 +714,14 @@ void Update(void)
 	if (state==NORMAL)
 		alarms = NORMAL;
 	else if (state==LOW)
-		alarms = LOW & NORMAL;
+		alarms = NORMAL | LOW;
 	else if (state==CRITICAL)
-		alarms = CRITICAL & LOW & NORMAL;
+		alarms = NORMAL | LOW | CRITICAL;
 
 	/* check AC status. */
 
-	if (SciACPower()!=power) {
+	i = SciACPower();
+	if ((i!=power) && (i!=0)) {
 		if (power==SCI_MAINS) {
 			power = SCI_BATTERY;
 			if (selectbay==EMPTY) {
@@ -543,33 +731,26 @@ void Update(void)
 				copyXPMArea(62, 93, 15, 10, 44, 33);
 			}
 			PowerChanged(power, config);
-			printf("BATTERY\n");
 		} else {
 			power = SCI_MAINS;
 			if (selectbay!=EMPTY)
 				selectbay = BOTH;
 			copyXPMArea(28, 93, 15, 10, 44, 33);
 			PowerChanged(power, config);
-			printf("MAINS\n");
 		}
 	}
 
 	/* paste up the default charge status and time */
 
-	copyXPMArea(104,  6,  5, 7, 6, 7);
+	copyXPMArea(119, 15, 5, 7, 6, 7);
 	copyXPMArea(83, 81, 41, 9, 15, 7);
 
-	/* check to see if we are charging */
+	/* check to see if the batteries are charging */
 
-/*	if ((int)(my_cur_info.battery_status)==3) {
-		copyXPMArea(98,  6, 5, 7,  6,  7);
-		copyXPMArea(75, 81, 1, 2, 17,  9);
-		copyXPMArea(75, 81, 1, 2, 17, 12);
-	}*/
-
-	/* repaint buttons. */
-
-	copyXPMArea(66, 56, 52, 11, 6, 48);
+	if (BatteryCharging()==3) {
+		copyXPMArea(119, 5, 5, 7,  6, 7);
+		copyXPMArea(83, 69, 5, 9, 15, 7);
+	}
 
 	/* paste up the "Time Left", this time means:
 	     If not charging: Time left before battery drains to 0% 
@@ -578,7 +759,7 @@ void Update(void)
 	if (time<0) {
 		/* dodgy time so just display a "null" indicator (--:--) */
 
-		copyXPMArea(83, 81, 41, 9, 15, 7);
+		copyXPMArea(83, 69, 41, 9, 15, 7);
 	} else {
 		copyXPMArea((hour/10)*7+5, 81, 7, 9, 21, 7);
 		copyXPMArea((hour%10)*7+5, 81, 7, 9, 29, 7);
@@ -643,6 +824,43 @@ void Update(void)
 }
 
 
+/*
+ * Determine if configuration information has changed and needs to be re-read
+ */
+void HandleDeadChild(long pid, int status)
+{
+	/* I don't quite understand the values here but it works */
+
+	if (status==25600) {
+		/* reread the alarm settings */
+		SetupAlarms(config);
+		Update();
+	}
+
+	if (status==51200) {
+		/* set the battery save options */
+		PowerChanged(power, config);
+	}
+
+	return;
+}
+
+
+/*
+ * This function is called from the main loop. It's job is to empty the
+ * ring buffers filled by the SIGCHLD signal handler
+ */
+void ShovelDeadChildren()
+{
+	while (phead!=ptail) {
+		HandleDeadChild(pid[ptail], retstatus[ptail]);
+		ptail = (ptail==PMAX-1) ? 0 : ++ptail;
+	}
+
+	return;
+}
+
+
 /*
  * the entry point of wmTuxTime
  */
@@ -655,16 +873,6 @@ int main(int argc, char *argv[])
 	struct passwd *pw;
 	struct stat info;
 
-	/* get the necessary I/O permissions */
-
-	if (ioperm(0xb2, 1, 1)) {
-		fprintf(stderr, "tuxtime: can't get I/O permissions.\n");
-		return 1;
-	}
-
-	/* development hack */
-	SciCloseInterface();
-
 	/* do some quick checks on the laptop */
 
 	if (SciSupportCheck(&version)==1) {
@@ -684,6 +892,99 @@ int main(int argc, char *argv[])
 		return 1;
 	}
 
+        /* drop root priveleges to minimize the risk of running suid root */
+
+        seteuid(getuid());
+        setegid(getgid());
+
+	switch (id) {
+		case 0xfc0f: /* Satellite 2060CDS/CDT */
+		case 0xfc10: /* Satellite 2550/2590 */
+		case 0xfc11: /* Portage 3110CT */
+		case 0xfc12: /* Portage 3300CT */
+		case 0xfc13: /* Portage 7020CT */
+		case 0xfc15: /* Satellite 4030/4030X/4050/4060/4070/4080/4090/4100X CDS/CDT */
+		case 0xfc17: /* Satellite 2520/2540 CDS/CDT */
+		case 0xfc18: /* Satellite 4000/4010 XCDT */
+		case 0xfc19: /* Satellite 4000/4010/4020 CDS/CDT */
+		case 0xfc1a: /* Tecra 8000x */
+		case 0xfc1c: /* Satellite2510CDS/CDT */
+		case 0xfc1f: /* Portage 7000CT/7010CT */
+		case 0xfc39: /* T2200SX */
+		case 0xfc40: /* T4500C */
+		case 0xfc41: /* T4500 */
+		case 0xfc45: /* T4400SX/SXC */
+		case 0xfc5f: /* T3300SL */
+		case 0xfc69: /* T1900C */
+		case 0xfc6a: /* T1900 */
+		case 0xfc6d: /* T1850C */
+		case 0xfc6e: /* T1850 */
+		case 0xfc6f: /* T1800 */
+		case 0xfc7e: /* T4600C */
+		case 0xfc7f: /* T4600 */
+		case 0xfc8a: /* T6600C */
+		case 0xfc91: /* T2400CT */
+		case 0xfc97: /* T4800CT */
+		case 0xfc99: /* T4700CS */
+		case 0xfc9b: /* T4700CT */
+		case 0xfc9d: /* T1950 */
+		case 0xfc9e: /* T3400/T3400CT */
+		case 0xfcb2: /* Libretto 30CT */
+		case 0xfcba: /* T2150 */
+		case 0xfcbe: /* T4850CT */
+		case 0xfcc0: /* Satellite Pro 420x */
+		case 0xfcc1: /* Satellite 100x */
+		case 0xfcc3: /* Tecra 710x/720x */
+		case 0xfcc6: /* Satellite Pro 410x */
+		case 0xfcca: /* Satellite Pro 400x */
+		case 0xfccb: /* Portage 610CT */
+		case 0xfccc: /* Tecra 700x */
+		case 0xfccf: /* T4900CT */
+		case 0xfcd0: /* Satellite 300x */
+		case 0xfcd1: /* Tecra 750CDT */
+		case 0xfcd2: /* Vision Connect -- what is this??? */
+		case 0xfcd4: /* Tecra 510x */
+		case 0xfcd5: /* Satellite 200x */
+		case 0xfcd7: /* Satellite Pro 430x */
+		case 0xfcd8: /* Tecra 740x */
+		case 0xfcd9: /* Portage 660CDT */
+		case 0xfcda: /* Tecra 730x/730X */
+		case 0xfcdb: /* Portage 620CT */
+		case 0xfcdc: /* Portage 650CT */
+		case 0xfcdd: /* Satellite 110x */
+		case 0xfcdf: /* Tecra 500x */
+		case 0xfce0: /* Tecra 780DVD */
+		case 0xfce2: /* Satellite 300x */
+		case 0xfce3: /* Satellite 310x */
+		case 0xfce4: /* Satellite Pro 490x */
+		case 0xfce5: /* Libretto 100CT */
+		case 0xfce6: /* Libretto 70CT */
+		case 0xfce7: /* Tecra 540x/550x */
+		case 0xfce8: /* Satellite Pro 470x/480x */
+		case 0xfce9: /* Tecra 750DVD */
+		case 0xfcea: /* Libretto 60 */
+		case 0xfceb: /* Libretto 50CT */
+		case 0xfcec: /* Satellite 320x/330x, Satellite 2500CDS */
+		case 0xfced: /* Tecra 520x/530x */
+		case 0xfcef: /* Satellite 220x, Satellite Pro 440x/460x */
+			printf("wmtuxtime: machine id: 0x%04x    BIOS version:"
+				" %d.%d    SCI version: %d.%d\n", id,
+				(bios & 0xff00)>>8, bios & 0xff,
+				(version & 0xff00)>>8, version & 0xff);
+			break;
+		default:
+			printf("wmtuxtime: unrecognized machine "
+				"identification:\n");
+			printf("machine id : 0x%04x    BIOS version : %d.%d"
+				"    SCI version: %d.%d\n", id,
+				(bios & 0xff00)>>8, bios & 0xff,
+				(version & 0xff00)>>8, version & 0xff);
+			printf("\nplease email this information to "
+				"jonathan@buzzard.org.uk and include the "
+				"model\nand model number, eg. Libretto 50CT "
+				" model number PA1249E\n");
+	}
+
 	/* check to see if a copy of wmTuxTime is already running */
 
 	if (!access(_PATH_VARRUN PID_FILE, R_OK)) {
@@ -699,7 +1000,7 @@ int main(int argc, char *argv[])
 				exit(1);
 			}
 
-			fprintf(stderr, "tuxtime: process %d appears to have "
+			fprintf(stderr, "wmtuxtime: process %d appears to have "
 				"died, continuing\n", pid);
 			unlink(_PATH_VARRUN PID_FILE);
 		}
@@ -718,14 +1019,14 @@ int main(int argc, char *argv[])
 	pw = getpwuid(getuid());
 	config = (char *) malloc(strlen(pw->pw_dir)+12);
 	if (config==NULL) {
-		fprintf(stderr, "tuxtime: unable to allocate sufficent memory, "
-			"exiting\n");
+		fprintf(stderr, "wmtuxtime: unable to allocate sufficent "
+			"memory, exiting\n");
 		exit(1);
 		}
 	strcpy(config, pw->pw_dir);
 	strcat(config, "/.tuxtimerc");
 
-	/* test to see if tuxtimerc file exists and if no create one */
+	/* test to see if tuxtimerc file exists and if not create one */
 
 	if ((stat(config, &info)!=0) && (errno==ENOENT)) {
 		CreateDefaultRC(config);
@@ -735,7 +1036,7 @@ int main(int argc, char *argv[])
 
 	HandleSignals();
 
-	/* read the config file and initilize state*/
+	/* read the config file and initilize state */
 
 	SetupAlarms(config);
 	alarms = NORMAL;
@@ -774,13 +1075,18 @@ int main(int argc, char *argv[])
 					RedrawWindow();
 					break;
 				case ButtonPress:
-					pressEvent(&event.xbutton);
+					PressEvent(&event.xbutton);
 					break;
 				case ButtonRelease:
+					ReleaseEvent(&event.xbutton);
                         		break;
 			}
 		}
 
+		/* handle dead child processes */
+
+		ShovelDeadChildren();
+
 		/* Redraw and wait for next update  */
 
 		RedrawWindow();
